@@ -21,17 +21,32 @@ from urllib2 import HTTPError
 
 from django.utils import simplejson
 from django.contrib.auth import authenticate
+from django.http import HttpResponse
+from django.template import TemplateDoesNotExist, RequestContext, loader
 
 from social_auth.backends import BaseOAuth2, OAuthBackend, USERNAME
-from social_auth.utils import sanitize_log_data, backend_setting, setting, \
-                              log, dsa_urlopen
-from social_auth.exceptions import AuthException, AuthCanceled, AuthFailed, \
-                                   AuthTokenError, AuthUnknownError
+from social_auth.utils import sanitize_log_data, backend_setting, setting,\
+    log, dsa_urlopen
+from social_auth.exceptions import AuthException, AuthCanceled, AuthFailed,\
+    AuthTokenError, AuthUnknownError
 
 
 # Facebook configuration
 FACEBOOK_ME = 'https://graph.facebook.com/me?'
 ACCESS_TOKEN = 'https://graph.facebook.com/oauth/access_token?'
+USE_APP_AUTH = setting('FACEBOOK_APP_AUTH', False)
+LOCAL_HTML = setting('FACEBOOK_LOCAL_HTML', 'facebook.html')
+APP_NAMESPACE = setting('FACEBOOK_APP_NAMESPACE', None)
+REDIRECT_HTML = """
+<script type="text/javascript">
+    var domain = 'https://apps.facebook.com/',
+        redirectURI = domain + {{ FACEBOOK_APP_NAMESPACE }} + '/';
+    window.top.location = 'https://www.facebook.com/dialog/oauth/' +
+    '?client_id={{ FACEBOOK_APP_ID }}' +
+    '&redirect_uri=' + encodeURIComponent(redirectURI) +
+    '&scope={{ FACEBOOK_EXTENDED_PERMISSIONS }}';
+</script>
+"""
 
 
 class FacebookBackend(OAuthBackend):
@@ -58,6 +73,7 @@ class FacebookAuth(BaseOAuth2):
     RESPONSE_TYPE = None
     SCOPE_SEPARATOR = ','
     AUTHORIZATION_URL = 'https://www.facebook.com/dialog/oauth'
+    ACCESS_TOKEN_URL = ACCESS_TOKEN
     SETTINGS_KEY_NAME = 'FACEBOOK_APP_ID'
     SETTINGS_SECRET_NAME = 'FACEBOOK_API_SECRET'
     SCOPE_VAR_NAME = 'FACEBOOK_EXTENDED_PERMISSIONS'
@@ -96,8 +112,10 @@ class FacebookAuth(BaseOAuth2):
             url = ACCESS_TOKEN + urlencode({
                 'client_id': backend_setting(self, self.SETTINGS_KEY_NAME),
                 'redirect_uri': self.get_redirect_uri(state),
-                'client_secret': backend_setting(self,
-                                                 self.SETTINGS_SECRET_NAME),
+                'client_secret': backend_setting(
+                    self,
+                    self.SETTINGS_SECRET_NAME
+                ),
                 'code': self.data['code']
             })
             try:
@@ -111,28 +129,42 @@ class FacebookAuth(BaseOAuth2):
                 expires = response['expires'][0]
 
         if 'signed_request' in self.data:
-            response = load_signed_request(self.data.get('signed_request'),
-                                           backend_setting(
-                                               self,
-                                               self.SETTINGS_SECRET_NAME))
+            response = load_signed_request(
+                self.data.get('signed_request'),
+                backend_setting(self, self.SETTINGS_SECRET_NAME)
+            )
 
             if response is not None:
-                access_token = response.get('access_token') or \
-                               response.get('oauth_token') or \
+                access_token = response.get('access_token') or\
+                               response.get('oauth_token') or\
                                self.data.get('access_token')
 
                 if 'expires' in response:
                     expires = response['expires']
 
         if access_token:
-            return self.do_auth(access_token, expires=expires,
-                                *args, **kwargs)
+            return self.do_auth(access_token, expires=expires, *args, **kwargs)
         else:
             if self.data.get('error') == 'access_denied':
                 raise AuthCanceled(self)
             else:
                 raise AuthException(self)
 
+    @classmethod
+    def process_refresh_token_response(cls, response):
+        return dict((key, val[0])
+                        for key, val in cgi.parse_qs(response).iteritems())
+
+    @classmethod
+    def refresh_token_params(cls, token):
+        client_id, client_secret = cls.get_key_and_secret()
+        return {
+            'fb_exchange_token': token,
+            'grant_type': 'fb_exchange_token',
+            'client_id': client_id,
+            'client_secret': client_secret
+        }
+
     def do_auth(self, access_token, expires=None, *args, **kwargs):
         data = self.user_data(access_token)
 
@@ -158,7 +190,7 @@ class FacebookAuth(BaseOAuth2):
     @classmethod
     def enabled(cls):
         """Return backend enabled status by checking basic settings"""
-        return backend_setting(cls, cls.SETTINGS_KEY_NAME) and \
+        return backend_setting(cls, cls.SETTINGS_KEY_NAME) and\
                backend_setting(cls, cls.SETTINGS_SECRET_NAME)
 
 
@@ -179,18 +211,82 @@ def load_signed_request(signed_request, api_secret=None):
         data = simplejson.loads(base64_url_decode(payload))
 
         expected_sig = hmac.new(api_secret or setting('FACEBOOK_API_SECRET'),
-                                msg=payload,
-                                digestmod=hashlib.sha256).digest()
+            msg=payload,
+            digestmod=hashlib.sha256).digest()
 
         # allow the signed_request to function for upto 1 day
         if sig == expected_sig and \
-                data[u'issued_at'] > (time.time() - 86400):
+           data[u'issued_at'] > (time.time() - 86400):
             return data
     except ValueError:
         pass  # ignore if can't split on dot
 
 
+class FacebookAppAuth(FacebookAuth):
+    """Facebook Application Authentication support"""
+    uses_redirect = False
+
+    def auth_complete(self, *args, **kwargs):
+        if not self.application_auth():
+            return HttpResponse(self.auth_html())
+
+        access_token = None
+        expires = None
+
+        if 'signed_request' in self.data:
+            response = load_signed_request(
+                self.data.get('signed_request'),
+                backend_setting(self, self.SETTINGS_SECRET_NAME)
+            )
+
+            if response is not None:
+                access_token = response.get('access_token') or\
+                               response.get('oauth_token') or\
+                               self.data.get('access_token')
+
+                if 'expires' in response:
+                    expires = response['expires']
+
+        if access_token:
+            return self.do_auth(access_token, expires=expires, *args, **kwargs)
+        else:
+            if self.data.get('error') == 'access_denied':
+                raise AuthCanceled(self)
+            else:
+                raise AuthException(self)
+
+    def application_auth(self):
+        required_params = ('user_id', 'oauth_token')
+        data = load_signed_request(
+            self.data.get('signed_request'),
+            backend_setting(self, self.SETTINGS_SECRET_NAME)
+        )
+        for param in required_params:
+            if not param in data:
+                return False
+        return True
+
+    def auth_html(self):
+        app_id = backend_setting(self, self.SETTINGS_KEY_NAME)
+        ctx = {
+            'FACEBOOK_APP_ID':  app_id,
+            'FACEBOOK_EXTENDED_PERMISSIONS': ','.join(
+                backend_setting(self, self.SCOPE_VAR_NAME)
+            ),
+            'FACEBOOK_COMPLETE_URI': self.redirect_uri,
+            'FACEBOOK_APP_NAMESPACE': APP_NAMESPACE or app_id
+        }
+
+        try:
+            fb_template = loader.get_template(LOCAL_HTML)
+        except TemplateDoesNotExist:
+            fb_template = loader.get_template_from_string(REDIRECT_HTML)
+        context = RequestContext(self.request, ctx)
+
+        return fb_template.render(context)
+
+
 # Backend definition
 BACKENDS = {
-    'facebook': FacebookAuth,
+    'facebook': FacebookAppAuth if USE_APP_AUTH else FacebookAuth,
 }
