@@ -169,12 +169,10 @@ def backend_setting(backend, name, default=None):
         2. Search for setting given by name
         3. Return default
     """
-    backend_setting_name = '%s_%s' % (
-        backend.AUTH_BACKEND.name.upper().replace('-', '_'),
-        name
-    )
-    if hasattr(settings, backend_setting_name):
-        return setting(backend_setting_name)
+    backend_name = get_backend_name(backend)
+    setting_name = '%s_%s' % (backend_name.upper().replace('-', '_'), name)
+    if hasattr(settings, setting_name):
+        return setting(setting_name)
     elif hasattr(settings, name):
         return setting(name)
     else:
@@ -223,14 +221,6 @@ def clean_partial_pipeline(request):
         request.session.pop(name, None)
 
 
-def log_exceptions_to_messages(request, backend, err):
-    """Log exception messages to messages app if it's installed."""
-    if 'django.contrib.messages' in setting('INSTALLED_APPS'):
-        from django.contrib.messages.api import error
-        name = backend.AUTH_BACKEND.name
-        error(request, unicode(err), extra_tags='social-auth %s' % name)
-
-
 def url_add_parameters(url, params):
     """Adds parameters to URL, parameter will be repeated if already present"""
     if params:
@@ -264,6 +254,10 @@ def dsa_urlopen(*args, **kwargs):
     return urlopen(*args, **kwargs)
 
 
+def get_backend_name(backend):
+    return getattr(getattr(backend, 'AUTH_BACKEND', backend), 'name', None)
+
+
 if __name__ == '__main__':
     import doctest
     doctest.testmod()
